"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const prisma_schema_parser_1 = require("@loancrate/prisma-schema-parser");
const internals_1 = require("@prisma/internals");
const promises_1 = require("fs/promises");
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("util"));
const handler = {
    onManifest() {
        return {
            version: '1',
            defaultOutput: './schema.ts',
            requiresGenerators: ['prisma-client-js'],
        };
    },
    async onGenerate(options) {
        const { output, config } = options.generator;
        const sortByName = config.sortByName === 'true';
        const includeRaw = config.includeRaw === 'true';
        const outputFilePath = (0, internals_1.parseEnvValue)(output);
        const extension = path_1.default.extname(outputFilePath);
        if (!['.ts', '.js', '.json'].includes(extension)) {
            throw new Error(`Invalid extension ${extension} for output file ${outputFilePath}. Use '.ts', '.js' or '.json'`);
        }
        const schema = options.datamodel;
        const astByTypeAndName = (0, prisma_schema_parser_1.parsePrismaSchema)(schema)
            .declarations.filter((o) => ['enum', 'type', 'model'].includes(o.kind))
            .reduce((acc, o) => {
            acc[o.kind][o.name.value] = {
                ...o,
                attributes: o.members
                    .filter((f) => f.kind === 'blockAttribute')
                    .map((a) => (0, prisma_schema_parser_1.formatAst)(a)),
                fields: o.members
                    .filter((f) => f.kind === 'field')
                    .reduce((acc, f) => {
                    var _a, _b, _c, _d, _e;
                    acc[f.name.value] = {
                        name: (_a = f.name) === null || _a === void 0 ? void 0 : _a.value,
                        type: (_c = (_b = f.type) === null || _b === void 0 ? void 0 : _b.name) === null || _c === void 0 ? void 0 : _c.value,
                        comment: (_d = f.comment) === null || _d === void 0 ? void 0 : _d.text,
                        attributes: (_e = f.attributes) === null || _e === void 0 ? void 0 : _e.map((a) => (0, prisma_schema_parser_1.formatAst)(a)),
                    };
                    return acc;
                }, {}),
            };
            return acc;
        }, { model: {}, type: {}, enum: {} });
        let datamodel = {
            ...options.dmmf.datamodel,
            types: options.dmmf.datamodel.types.map((t) => ({
                ...t,
                raw: includeRaw
                    ? (0, prisma_schema_parser_1.formatAst)(astByTypeAndName['type'][t.name])
                    : undefined,
            })),
            enums: options.dmmf.datamodel.enums.map((e) => ({
                ...e,
                raw: includeRaw
                    ? (0, prisma_schema_parser_1.formatAst)(astByTypeAndName['enum'][e.name])
                    : undefined,
            })),
            models: options.dmmf.datamodel.models.map((m) => ({
                ...m,
                attributes: astByTypeAndName['model'][m.name].attributes,
                fields: m.fields.map((f) => ({
                    ...f,
                    comment: astByTypeAndName['model'][m.name].fields[f.name].comment,
                    attributes: astByTypeAndName['model'][m.name].fields[f.name].attributes,
                })),
                raw: includeRaw
                    ? (0, prisma_schema_parser_1.formatAst)(astByTypeAndName['model'][m.name])
                    : undefined,
            })),
        };
        if (sortByName) {
            datamodel = {
                types: datamodel.types.reduce((acc, t) => {
                    acc[t.name] = t;
                    return acc;
                }, {}),
                enums: datamodel.enums.reduce((acc, e) => {
                    acc[e.name] = e;
                    return acc;
                }, {}),
                models: datamodel.models.reduce((acc, m) => {
                    acc[m.name] = {
                        ...m,
                        fields: m.fields.reduce((acc, f) => {
                            acc[f.name] = f;
                            return acc;
                        }, {}),
                    };
                    return acc;
                }, {}),
            };
        }
        await (0, promises_1.mkdir)(path_1.default.dirname(outputFilePath), { recursive: true });
        await (0, promises_1.writeFile)(outputFilePath, extension === '.json'
            ? JSON.stringify(datamodel, null, 2)
            : extension === '.ts'
                ? `
// auto-generated file (do not edit)
import { type Prisma } from "@prisma/client";

export type PrismaFieldEnhanced = Prisma.DMMF.Field & { attributes: string[] };

export default ${util_1.default.inspect(datamodel, false, 10)}`
                : `
// auto-generated file (do not edit)
module.exports = ${util_1.default.inspect(datamodel, false, 10)}`);
    },
};
exports.default = handler;
//# sourceMappingURL=handler.js.map